package jbitarrays;

import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import javax.annotation.concurrent.NotThreadSafe;

import jbitarrays.api.CrawlerReader;
import jbitarrays.api.CrawlerWriter;
import jbitarrays.api.InfiniteBitArray;

/**
 * @see JBitArrays#startReading(InfiniteBitArray, int)
 */
@NotThreadSafe final class DefaultCrawlerReader implements CrawlerReader {
	private final InfiniteBitArray array;
	private int index;

	public DefaultCrawlerReader(@Nonnull InfiniteBitArray array) {
		this(array, 0);
	}

	DefaultCrawlerReader(@Nonnull InfiniteBitArray array, @Nonnegative int index) {
		if (array == null || index < 0) {
			throw new IllegalArgumentException("array != null && index >= 0");
		}
		this.array = array;
		this.index = index;
	}

	public boolean getNext() {
		return array.get(index++);
	}

	public CrawlerWriter toWriter() {
		return new DefaultCrawlerWriter(array, index);
	}

	public int getNextN(int howMany) {
		Utils.checkHowMany(howMany);
		int ret = array.getN(index, howMany);
		index += howMany;
		return ret;
	}

	@Override public CrawlerReader clone() {
		return new DefaultCrawlerReader(array, index);
	}
}
